def apply_defaults(settings):
    from datetime import timedelta

    settings.AUTHENTICATION_BACKENDS = (
        "django.contrib.auth.backends.ModelBackend",
        "allauth.account.auth_backends.AuthenticationBackend",
    )

    settings.INSTALLED_APPS = ("rest_framework_simplejwt.token_blacklist",)
    settings.SIMPLE_JWT = {
        "ACCESS_TOKEN_LIFETIME": timedelta(minutes=15),
        "REFRESH_TOKEN_LIFETIME": timedelta(days=7),
        "ROTATE_REFRESH_TOKENS": True,
        "BLACKLIST_AFTER_ROTATION": True,
    }
    settings.ACCOUNT_AUTHENTICATION_METHOD = "email"
    settings.ACCOUNT_EMAIL_REQUIRED = True
    settings.ACCOUNT_USERNAME_REQUIRED = False

    settings.REST_FRAMEWORK = {
        "DEFAULT_AUTHENTICATION_CLASSES": (
            "rest_framework_simplejwt.authentication.JWTAuthentication",
        ),
    }



from django.contrib.auth.models import AbstractUser
from django.db import models
from ..managers.users import UserManager

class BunifuAbstractUser(AbstractUser):
    """
    This a s abstract user model so the user can decide
    to exitent the django model with it
    """
    username = None
    email = models.EmailField(unique=True)

    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = []
    
    objects = UserManager()
    
    class Meta:
        abstract = True


    def __str__(self):
        return self.email





DEBUG = True

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "bunifu_django_auth",
    "rest_framework",
    "rest_framework_simplejwt",
    'rest_framework_simplejwt.token_blacklist',
    "allauth",
    "allauth.account",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "allauth.account.middleware.AccountMiddleware",
]

ROOT_URLCONF = "bunifu_django_auth.urls"

SECRET_KEY = "test-secret-key"

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}

AUTH_USER_MODEL = "bunifu_django_auth.User"
REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": (
        "rest_framework_simplejwt.authentication.JWTAuthentication",
    )
}



import factory
from django.contrib.auth import get_user_model

User = get_user_model()


class UserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = User

    email = factory.Faker("email")
    username = factory.Faker("user_name")
    password = factory.PostGenerationMethodCall("set_password", "password123")


from django.apps import AppConfig
from django.conf import settings
from datetime import timedelta


class BunifuDjangoAuthConfig(AppConfig):
    name = "bunifu_django_auth"
    verbose_name = "Bunifu Django Auth"

    def ready(self):
        self.apply_default_settings()
        from . import models  # noqa: F401

    def apply_default_settings(self):
        """
        Apply safe defaults without overriding user-defined settings.
        """

        # ----------------------------
        # AUTHENTICATION BACKENDS
        # ----------------------------
        default_backends = [
            "django.contrib.auth.backends.ModelBackend",
            "allauth.account.auth_backends.AuthenticationBackend",
        ]

        existing_backends = list(
            getattr(settings, "AUTHENTICATION_BACKENDS", [])
        )

        for backend in default_backends:
            if backend not in existing_backends:
                existing_backends.append(backend)

        settings.AUTHENTICATION_BACKENDS = existing_backends

        # ----------------------------
        # SIMPLE JWT
        # ----------------------------
        default_jwt = {
            "ACCESS_TOKEN_LIFETIME": timedelta(minutes=15),
            "REFRESH_TOKEN_LIFETIME": timedelta(days=7),
            "ROTATE_REFRESH_TOKENS": True,
            "BLACKLIST_AFTER_ROTATION": True,
        }

        project_jwt = getattr(settings, "SIMPLE_JWT", {})
        merged_jwt = {**default_jwt, **project_jwt}

        settings.SIMPLE_JWT = merged_jwt

        # ----------------------------
        # REST FRAMEWORK
        # ----------------------------
        default_auth_class = (
            "rest_framework_simplejwt.authentication.JWTAuthentication",
        )

        rf_settings = getattr(settings, "REST_FRAMEWORK", {})

        existing_auth_classes = rf_settings.get(
            "DEFAULT_AUTHENTICATION_CLASSES",
            (),
        )

        if not existing_auth_classes:
            rf_settings["DEFAULT_AUTHENTICATION_CLASSES"] = default_auth_class

        settings.REST_FRAMEWORK = rf_settings




from rest_framework import serializers
from django.contrib.auth import get_user_model

User = get_user_model()


class RegisterSerializer(serializers.ModelSerializer):
    password = serializers.CharField(write_only=True)

    class Meta:
        model = User
        fields = ["email", "password",]

    def create(self, validated_data):
        return User.objects.create_user(**validated_data)






import pytest
from rest_framework.test import APIClient
from ..factories.users import UserFactory

@pytest.mark.django_db
def test_logout_blacklists_refresh_token():
    raw_password = "password123"
    user = UserFactory(password=raw_password)
    client = APIClient()
    login_resp = client.post(
        "/login/",
        {"email": user.email, "password": raw_password},
        format="json"
    )
    assert login_resp.status_code == 200
    refresh_token = login_resp.data["refresh"]
    access_token = login_resp.data["access"]

    client.credentials(HTTP_AUTHORIZATION=f"Bearer {access_token}")

    logout_resp = client.post(
        "/logout/",
        {"refresh": refresh_token},
        format="json"
    )
    
    assert logout_resp.status_code == 200
